/*
 * Created on 2005-maj-17
 */
package com.eviware.soapui.impl.wsdl;

import java.util.HashSet;
import java.util.Set;

import javax.swing.ImageIcon;

import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.impl.wsdl.actions.teststep.DeleteTestStepAction;
import com.eviware.soapui.impl.wsdl.actions.teststep.RenameTestStepAction;
import com.eviware.soapui.model.support.AbstractModelItem;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.tree.nodes.TestStepTreeNode;

/**
 * Base class for WSDL project test steps.
 *  
 * @author Ole.Matzura
 */

abstract public class WsdlTestStep extends AbstractModelItem implements TestStep
{
   private final WsdlTestCase testCase;
   private TestStepConfig config;
   
   private final Set<TestStepListener> listeners = new HashSet<TestStepListener>();

   protected WsdlTestStep( WsdlTestCase testCase, TestStepConfig config )
   {
      this.testCase = testCase;
      this.config = config;
      
      addAction( new DeleteTestStepAction( this ) );
      addAction( new RenameTestStepAction( this ) );
   }

   public TestCase getTestCase()
   {
      return testCase;
   }
   
   public String getName()
   {
      try
      {
         return config.getName();
      }
      catch (Exception e)
      {
         return null;
      }
   }
   
   public TestStepConfig getConfig()
   {
      return config;
   }
   
   /**
    * Called from WsdlTestCase when moving a teststep due to no move functionality
    * in xmlbeans generated arrays. 
    * 
    * @param config the new config to use, will be a copy of the existing one. The current
    * will be invalid
    */
   
   protected void resetConfigOnMove( TestStepConfig config )
   {
   	this.config = config;
   }

   public void setName(String name)
   {
      String old = getName();
      config.setName( name );
      notifyPropertyChanged( NAME_PROPERTY, old, name );
   }
   
   protected SoapUITreeNode createTreeNode()
   {
      return new TestStepTreeNode( this );
   }

	public void addTestStepListener( TestStepListener listener )
	{
		listeners.add( listener );
	}
	
	public void removeTestStepListener( TestStepListener listener )
	{
		listeners.remove( listener );
	}
	
	public void fireBeforeRun()
	{
		TestStepListener [] listeners = this.listeners.toArray( new TestStepListener[this.listeners.size()]);
		
		for (int i = 0; i < listeners.length; i++)
		{
			TestStepListener listener = listeners[i];
			listener.beforeRun( this );
		}
	}
	
	public void fireAfterRun()
	{
		TestStepListener [] listeners = this.listeners.toArray( new TestStepListener[this.listeners.size()]);
		
		for (int i = 0; i < listeners.length; i++)
		{
			TestStepListener listener = listeners[i];
			listener.afterRun( this );
		}
	}

	public void prepare(TestRunner testRunner)
	{
	}

	public ImageIcon getIcon()
	{
		return null;
	}
}
