/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.iface;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.model.iface.Operation;

/**
 * Removes a WsdlInterface from a WsdlProject
 * 
 * @author Ole.Matzura
 */

public class RemoveInterfaceAction extends AbstractAction
{
   private final WsdlInterface iface;

	public RemoveInterfaceAction( WsdlInterface iface )
   {
      super( "Remove" );
		this.iface = iface;
      putValue( Action.SHORT_DESCRIPTION, "Removes this interface from the project" );
   }
   
   
   public void actionPerformed(ActionEvent e)
	{
      if( JOptionPane.OK_OPTION != JOptionPane.showConfirmDialog( SoapUI.getInstance().getFrame(), 
            "Remove interface [" + iface.getName() + "] from project [" + 
            iface.getProject().getName() + "]?", "Remove Interface", JOptionPane.OK_CANCEL_OPTION ))
      {
         return;
      }

      WsdlProject project = (WsdlProject) iface.getProject();
      
      for( int c = 0; c < iface.getOperationCount(); c++ )
      {
         Operation operation = iface.getOperationAt( c );
         for( int j = 0; j < operation.getRequestCount(); j++ )
         {
            SoapUI.getInstance().closeTab( operation.getRequestAt( j ));
         }
      }
      
      project.removeInterface( iface);
   }

}
