/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.project;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.model.iface.Operation;

/**
 * Removes a WsdlProject from the workspace
 * 
 * @author Ole.Matzura
 */

public class RemoveProjectAction extends AbstractAction
{
   private final WsdlProject project;

	public RemoveProjectAction( WsdlProject project)
   {
      super( "Remove" );
		this.project = project;
      putValue( Action.SHORT_DESCRIPTION, "Removes this project from the workspace" );
   }
   
   public void actionPerformed(ActionEvent e)
	{
      if( JOptionPane.OK_OPTION != JOptionPane.showConfirmDialog( SoapUI.getInstance().getFrame(), 
            "Remove project [" + project.getName() + "] from workspace?", "Remove Project", 
            JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE ))
      {
         return;
      }

      for( int x = 0; x < project.getInterfaceCount(); x++ )
      {
         WsdlInterface iface = (WsdlInterface) project.getInterfaceAt(x);
         
         for( int c = 0; c < iface.getOperationCount(); c++ )
         {
            Operation operation = iface.getOperationAt( c );
            for( int j = 0; j < operation.getRequestCount(); j++ )
            {
               SoapUI.getInstance().closeTab( operation.getRequestAt( j ));
            }
         }
      }
      
      SoapUI.getInstance().getWorkspace().removeProject( project );
   }

}
