/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.request;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.support.XmlUtils;

/**
 * Recreates a WsdlRequest from its WsdlOperations schema definition
 *  
 * @author Ole.Matzura
 */

public class RecreateRequestAction extends AbstractAction
{
   private final WsdlRequest request;

	public RecreateRequestAction( WsdlRequest request )
   {
      super( "Recreate request" );
		this.request = request;
      putValue( Action.SMALL_ICON, SoapUI.createImageIcon( "/recreate_request.gif"));
      putValue( Action.SHORT_DESCRIPTION, "Recreates a default request from the schema" );
   }
   
   public void actionPerformed(ActionEvent e)
   {
      boolean createOptional = JOptionPane.showConfirmDialog( SoapUI.getInstance().getFrame(), 
      		"Create optional elements in schema?", "Recreate Request", JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION;

      String req = ((WsdlOperation)request.getOperation()).createRequest( createOptional );
      
      if( request.getRequestContent() != null && request.getRequestContent().trim().length() > 0 )
   	{
         int option = JOptionPane.showConfirmDialog( SoapUI.getInstance().getFrame(),
        		 "Keep existing values", "Recreate Request", JOptionPane.YES_NO_OPTION );
        
         if( option == JOptionPane.YES_OPTION )
         {
        		req = XmlUtils.transferValues( request.getRequestContent(), req );
         }         	
   	}
      
      request.setRequest( req );
   }
}