/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request;

import javax.swing.JPanel;

import com.eviware.soapui.impl.EmptyPanelBuilder;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.model.DesktopPanel;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.support.PropertiesTable;

/**
 * PanelBuilder for WsdlRequest
 * 
 * @author Ole.Matzura
 */

public class WsdlRequestPanelBuilder extends EmptyPanelBuilder
{
   private final WsdlRequest request;

   public WsdlRequestPanelBuilder(WsdlRequest request)
   {
      this.request = request;
   }

   public DesktopPanel buildDesktopPanel()
   {
      return new WsdlRequestDesktopPanel(request);
   }

   public WsdlRequest getRequest()
   {
      return request;
   }

   public boolean canBuildDesktopPanel()
   {
      return true;
   }

   public String getDesktopPanelTitle()
   {
      Operation operation = request.getOperation();
      return operation.getInterface().getProject().getName() + "." + operation.getInterface().getName() + "." + 
         operation.getName() + ": " + request.getName();
   }
   
   public JPanel buildOverviewPanel()
   {
   	PropertiesTable table = new PropertiesTable( "Request Properties", request );
   	request.addPropertyChangeListener( table.getTableModel() );
   	
   	table.addProperty( "Name", "name" );
   	table.addProperty( "Encoding", "encoding" );
   	table.addProperty( "Endpoint", "endpoint" );
   	table.addProperty( "Username", "username", true );
   	table.addProperty( "Password", "password", true );
   	table.addProperty( "Domain", "domain", true );
   	
   	return table;
   }

   public boolean hasOverviewPanel()
   {
      return true;
   }
}
